function [sys,x0,str,ts] = baloo_inn(t,x,u,flag,d,Dt,J,Res,Laf,L,B)
% baloo_inn(t,x,u,flag,d,Dt,J,Res,Laf,L,B)
% This is a discrete S-function that computes the innovation term.
%
% S-FUNCTION PARAMETERS :  d (the delay in the computation of the innovation)
%                          Dt (the sample time of this S-function)
%                          J,Res,Laf,L,B  from the model parameters  
% S-FUNCTION INPUTS : 
%           [y      : the output of the process (dim 1)
%            z(t-d) : the d-delayed output of the observer  (dim 3)
%            V]     : the systen input  (dim 1)
%                         
% NOTE : Dt Must be a divider of d strictly smaller than d -> error

% Last modified by Elnycko on the 23/07/2007

switch flag,
  case 0
    [sys,x0,str,ts]=mdlInitializeSizes(u,d,Dt); % Initialization
  case 2
    sys = mdlUpdate(t,x,u,d,Dt,J,Res,Laf,L,B); % Calculate derivatives
  case 3
    sys = mdlOutputs(t,x,u,d,Dt); % Calculate outputs  
  case { 1, 4, 9 } % Unused flags
    sys = [];
  otherwise
    error(['Unhandled flag = ',num2str(flag)]); % Error handling
end
% End of csfunc.


function [sys,x0,str,ts] = mdlInitializeSizes(u,d,Dt)
if d/Dt-floor(d/Dt)>0
    error('Dt should divide d');
end,
if d==Dt
    error('Dt shouldn''t be equal to d');
end,

sizes = simsizes;
sizes.NumContStates  = 0;
sizes.NumDiscStates  = 2*(d/Dt)+2; % la sortie du systme et le controle st memorises + innovation
sizes.NumOutputs     = 1;
sizes.NumInputs      = 5;
sizes.DirFeedthrough = 0;     
sizes.NumSampleTimes = 1;     %systeme hybride: 2,  continu: 0, discret:1
sys = simsizes(sizes);
x0=[zeros((d/Dt)+1,1)      % x0 =[y() <- d/Dt + 1 elements
    zeros((d/Dt),1)        %      u() <- d/Dt  elements 
            0         ];   %      Innovation];
str = [];
%
% Initialize the array of sample times; in this example the sample 
% time is continuous, so set ts to 0 and its offset to 0.
%
ts = [Dt 0];
% End of mdlInitializeSizes.


function sys = mdlUpdate(t,x,u,d,Dt,J,Res,Laf,L,B);
n=d/Dt;
% rmq : x(1:n+1)=[y(t-d)....y(t)]
%       x(n+2:2*n+1)=[V(t-d)...V(t-d/Dt)]
sys(1:n)=x(2:n+1);
sys(n+1)=u(1);  % u(1) is the output of the process
if u(2)==0
    u(2)=0.1; %i.e to avoid zero divisions
end,
if u(3)==0
    u(3)=0.1;
end,

sys(n+1+(1:n-1))=x(n+1+(2:n));
sys((n+1)+n)=u(5);

yhat_zero=[u(2);u(2)*u(3);u(2)*u(4)]; %change of variables
[tout,yout] = ode45(@(t,z)[-(Laf/L)*z(2)-(Res/L)*z(1)+ x(n+1+ceil(t/Dt))/L;
                               -(1/J)*z(3)-(B/J)*z(2)+(Laf/J)*z(1)^3-(Res/L)*z(2)-(Laf/L)*(z(2)^2/z(1))+(x(n+1+ceil(t/Dt))/L)*(z(2)/z(1));
                               (x(n+1+ceil(t/Dt))/L)*(z(3)/z(1))-(Res/L)*z(3)-(Laf/L)*(z(2)*z(3)/z(1))],0:Dt:d,yhat_zero);
yhat=yout(:,1)';
Integrande=(sys(1:n+1)-yhat).^2;
sys(2*n+2)=trapz(Integrande)*Dt;
% End of mdlDerivatives.

function sys = mdlOutputs(t,x,u,d,Dt)
I=x(2*(d/Dt)+2);
sys=I;
% End of mdlOutputs.